@@ -102,7 +102,7 @@ $c->admin_email ='calendar-admin@example.com';
 * creation of calendar collections.
 * Default: true
 */
-// $c->collections_always_exist = true;
+$c->collections_always_exist = false;
 
 /**
 * The name of a user's "home" calendar. This will be created for each
@@ -124,6 +124,18 @@ $c->admin_email ='calendar-admin@example.com';
 // $c->default_relationships = array();
 
 
+/**
+* An array of fields on the usr record which should be set to specific
+* values when the users are created.
+* Default: none
+*/
+// $c->template_usr = array( 'active' => true,
+//                           'locale' => 'it_IT',
+//                           'date_format_type' => 'E',
+//                           'email_ok' => date('Y-m-d')
+//                         );
+
+
 /***************************************************************************
 *                                                                          *
 *                     External Authentication Sources                      *
@@ -284,22 +296,6 @@ $c->admin_email ='calendar-admin@example.com';
 */
 // $c->local_tzid;
 
-/**
-* default is false
-* If true, then events requested from someone other than the admmin or owner
-* of a calendar will not get any alarm for it.  Most client software also
-* allows you to configure calendar by calendar which ones allow alarms, which
-* is better really, but if you aren't using such software you might want
-* to set this to true.
-*
-* You should be aware that there are potentially problems when someone without
-* full access modifies an event, in which case any alarms will be lost!  It really
-* *is* better if you can control this client-side, in so many ways...
-*
-* @todo remove this capability when client software is more generally smarter.
-*/
-// $c->hide_alarm = false;
-
 /**
 * Many people want this, but it may be a security issue for you, so it is
 * disabled by default.  If you enable it, then confidential / private events
@@ -316,4 +312,4 @@ $c->admin_email ='calendar-admin@example.com';
 * If the only person that devious is your sysadmin then you probably already
 * enabled this option...
 */
-// $c->allow_get_email_visibility = false;
\ No newline at end of file
+// $c->allow_get_email_visibility = false;
