@@ -33,9 +33,9 @@ CREATE TYPE rrule_parts AS (
   freq TEXT,
   count INT,
   interval INT,
---  bysecond INT[],
---  byminute INT[],
---  byhour INT[],
+  bysecond INT[],
+  byminute INT[],
+  byhour INT[],
   bymonthday INT[],
   byyearday INT[],
   byweekno INT[],
@@ -61,20 +61,17 @@ BEGIN
   result.interval   := COALESCE(substring(repeatrule from 'INTERVAL=([0-9]+)(;|$)')::int, 1);
   result.wkst       := substring(repeatrule from 'WKST=(MO|TU|WE|TH|FR|SA|SU)(;|$)');
 
-  /**
-  * We can do the array conversion as a simple cast, since the strings are simple numbers, with no commas
-  */
-  result.byday    := ('{' || substring(repeatrule from 'BYDAY=(([+-]?[0-9]{0,2}(MO|TU|WE|TH|FR|SA|SU),?)+)(;|$)') || '}')::text[];
+  result.byday      := string_to_array( substring(repeatrule from 'BYDAY=(([+-]?[0-9]{0,2}(MO|TU|WE|TH|FR|SA|SU),?)+)(;|$)'), ',');
 
-  result.byyearday  := ('{' || substring(repeatrule from 'BYYEARDAY=([0-9,+-]+)(;|$)') || '}')::int[];
-  result.byweekno   := ('{' || substring(repeatrule from 'BYWEEKNO=([0-9,+-]+)(;|$)') || '}')::int[];
-  result.bymonthday := ('{' || substring(repeatrule from 'BYMONTHDAY=([0-9,+-]+)(;|$)') || '}')::int[];
-  result.bymonth    := ('{' || substring(repeatrule from 'BYMONTH=(([+-]?[0-1]?[0-9],?)+)(;|$)') || '}')::int[];
-  result.bysetpos   := ('{' || substring(repeatrule from 'BYSETPOS=(([+-]?[0-9]{1,3},?)+)(;|$)') || '}')::int[];
+  result.byyearday  := string_to_array(substring(repeatrule from 'BYYEARDAY=([0-9,+-]+)(;|$)'), ',');
+  result.byweekno   := string_to_array(substring(repeatrule from 'BYWEEKNO=([0-9,+-]+)(;|$)'), ',');
+  result.bymonthday := string_to_array(substring(repeatrule from 'BYMONTHDAY=([0-9,+-]+)(;|$)'), ',');
+  result.bymonth    := string_to_array(substring(repeatrule from 'BYMONTH=(([+-]?[0-1]?[0-9],?)+)(;|$)'), ',');
+  result.bysetpos   := string_to_array(substring(repeatrule from 'BYSETPOS=(([+-]?[0-9]{1,3},?)+)(;|$)'), ',');
 
---  result.bysecond   := list_to_array(substring(repeatrule from 'BYSECOND=([0-9,]+)(;|$)'))::int[];
---  result.byminute   := list_to_array(substring(repeatrule from 'BYMINUTE=([0-9,]+)(;|$)'))::int[];
---  result.byhour     := list_to_array(substring(repeatrule from 'BYHOUR=([0-9,]+)(;|$)'))::int[];
+  result.bysecond   := string_to_array(substring(repeatrule from 'BYSECOND=([0-9,]+)(;|$)'), ',');
+  result.byminute   := string_to_array(substring(repeatrule from 'BYMINUTE=([0-9,]+)(;|$)'), ',');
+  result.byhour     := string_to_array(substring(repeatrule from 'BYHOUR=([0-9,]+)(;|$)'), ',');
 
   RETURN result;
 END;
@@ -537,10 +534,18 @@ BEGIN
   loopcount := 0;
 
   SELECT * INTO rrule FROM parse_rrule_parts( basedate, repeatrule );
+
   IF rrule.count IS NOT NULL THEN
     loopmax := rrule.count;
   ELSE
-    loopmax := max_count;
+    -- max_count is pretty arbitrary, so we scale it somewhat here depending on the frequency.
+    IF rrule.freq = 'DAILY' THEN
+      loopmax := max_count * 20;
+    ELSIF rrule.freq = 'WEEKLY' THEN
+      loopmax := max_count * 10;
+    ELSE
+      loopmax := max_count;
+    END IF;
   END IF;
 
   current_base := basedate;
@@ -622,3 +627,84 @@ BEGIN
   RETURN QUERY SELECT d FROM rrule_event_instances_range( basedate, repeatrule, basedate, maxdate, 300 ) d;
 END;
 $$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
+
+
+------------------------------------------------------------------------------------------------------
+-- In most cases we just want to know if there *is* an event overlapping the range, so we have a
+-- specific function for that.  Note that this is *not* strict, and can be called with NULLs.
+------------------------------------------------------------------------------------------------------
+CREATE or REPLACE FUNCTION rrule_event_overlaps( TIMESTAMP WITH TIME ZONE, TIMESTAMP WITH TIME ZONE, TEXT, TIMESTAMP WITH TIME ZONE, TIMESTAMP WITH TIME ZONE )
+                                         RETURNS BOOLEAN AS $$
+DECLARE
+  dtstart ALIAS FOR $1;
+  dtend ALIAS FOR $2;
+  repeatrule ALIAS FOR $3;
+  in_mindate ALIAS FOR $4;
+  in_maxdate ALIAS FOR $5;
+  base_date TIMESTAMP WITH TIME ZONE;
+  mindate TIMESTAMP WITH TIME ZONE;
+  maxdate TIMESTAMP WITH TIME ZONE;
+BEGIN
+
+  IF dtstart IS NULL THEN
+    RETURN NULL;
+  END IF;
+  IF dtend IS NULL THEN
+    base_date := dtstart;
+  ELSE
+    base_date := dtend;
+  END IF;
+
+  IF in_mindate IS NULL THEN
+    mindate := current_date - '10 years'::interval;
+  ELSE
+    mindate := in_mindate;
+  END IF;
+
+  IF in_maxdate IS NULL THEN
+    maxdate := current_date + '10 years'::interval;
+  ELSE
+    -- If we add the duration onto the event, then an overlap occurs if dtend <= increased end of range.
+    maxdate := in_maxdate + (base_date - dtstart);
+  END IF;
+
+  IF repeatrule IS NULL THEN
+    RETURN (dtstart <= maxdate AND base_date >= mindate);
+  END IF;
+
+  SELECT d INTO mindate FROM rrule_event_instances_range( base_date, repeatrule, mindate, maxdate, 60 ) d LIMIT 1;
+  RETURN FOUND;
+
+END;
+$$ LANGUAGE 'plpgsql' IMMUTABLE;
+
+
+-- Create a composite type for the parts of the RRULE.
+DROP TYPE rrule_instance CASCADE;
+CREATE TYPE rrule_instance AS (
+  dtstart TIMESTAMP WITH TIME ZONE,
+  rrule TEXT,
+  instance TIMESTAMP WITH TIME ZONE
+);
+
+CREATE or REPLACE FUNCTION rrule_event_instances( TIMESTAMP WITH TIME ZONE, TEXT )
+                                         RETURNS SETOF rrule_instance AS $$
+DECLARE
+  basedate ALIAS FOR $1;
+  repeatrule ALIAS FOR $2;
+  maxdate TIMESTAMP WITH TIME ZONE;
+  current TIMESTAMP WITH TIME ZONE;
+  result rrule_instance%ROWTYPE;
+BEGIN
+  maxdate := current_date + '10 years'::interval;
+
+  result.dtstart := basedate;
+  result.rrule   := repeatrule;
+
+  FOR current IN SELECT d FROM rrule_event_instances_range( basedate, repeatrule, basedate, maxdate, 300 ) d LOOP
+    result.instance := current;
+    RETURN NEXT result;
+  END LOOP;
+
+END;
+$$ LANGUAGE 'plpgsql' IMMUTABLE STRICT;
