@@ -46,6 +46,7 @@ class CalDAVRequest
 
   /**
   * The 'principal' (user/resource/...) which this request seeks to access
+  * @var CalDAVPrincipal
   */
   var $principal;
 
@@ -88,6 +89,21 @@ class CalDAVRequest
 
     $this->raw_post = file_get_contents ( 'php://input');
 
+    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['request']) && $c->dbg['request']) ) {
+      /** Log the request headers */
+      $lines = apache_request_headers();
+      dbg_error_log( "LOG ", "***************** Request Header ****************" );
+      foreach( $lines AS $v ) {
+        dbg_error_log( "LOG headers", "-->%s", $v );
+      }
+      dbg_error_log( "LOG ", "******************** Request ********************" );
+      // Log the request in all it's gory detail.
+      $lines = preg_split( '#[\r\n]+#', $this->raw_post);
+      foreach( $lines AS $v ) {
+        dbg_error_log( "LOG request", "-->%s", $v );
+      }
+    }
+
     if ( isset($debugging) && isset($_GET['method']) ) {
       $_SERVER['REQUEST_METHOD'] = $_GET['method'];
     }
@@ -176,6 +192,7 @@ class CalDAVRequest
     if ( preg_match( $bad_chars_regex, $this->path ) ) {
       $this->DoResponse( 400, translate("The calendar path contains illegal characters.") );
     }
+    if ( strstr($this->path,'//') ) $this->path = preg_replace( '#//+#', '/', $this->path);
 
     $this->user_no = $session->user_no;
     $this->username = $session->username;
@@ -208,7 +225,7 @@ class CalDAVRequest
       if ( $row->dav_name == $this->path."/" ) {
         $this->path = $row->dav_name;
         dbg_error_log( "caldav", "Path is actually a collection - sending Content-Location header." );
-        header( "Content-Location: $this->path" );
+        header( "Content-Location: ".ConstructURL($this->path) );
       }
 
       $this->collection_id = $row->collection_id;
@@ -233,10 +250,23 @@ EOSQL;
         $this->collection = $row;
       }
     }
-    else if ( preg_match( '#/(\S+@\S+[.]\S+)/?$#', $this->path) ) {
+    else if ( preg_match( '#^((/[^/]+/)calendar-proxy-(read|write))/?[^/]*$#', $this->path, $matches ) ) {
+      $this->collection_type = 'proxy';
+      $this->_is_proxy_request = true;
+      $this->proxy_type = $matches[3];
+      $this->collection_path = $matches[1].'/';  // Enforce trailling '/'
+      if ( $this->collection_path == $this->path."/" ) {
+        $this->path .= '/';
+        dbg_error_log( "caldav", "Path is actually a (proxy) collection - sending Content-Location header." );
+        header( "Content-Location: ".ConstructURL($this->path) );
+      }
+    }
+    else if ( preg_match( '#^/(\S+@\S+[.]\S+)/?$#', $this->path) ) {
+      /** @TODO: we should deprecate this now that Evolution 2.27 can do scheduling extensions */
       $this->collection_id = -1;
       $this->collection_type = 'email';
       $this->collection_path = $this->path;
+      $this->_is_principal = true;
     }
     else if ( preg_match( '#^(/[^/]+)/?$#', $this->path, $matches) ) {
       $this->collection_id = -1;
@@ -246,7 +276,7 @@ EOSQL;
       if ( $this->collection_path == $this->path."/" ) {
         $this->path .= '/';
         dbg_error_log( "caldav", "Path is actually a collection - sending Content-Location header." );
-        header( "Content-Location: $this->path" );
+        header( "Content-Location: ".ConstructURL($this->path) );
       }
     }
     else if ( $this->path == '/' ) {
@@ -266,6 +296,38 @@ EOSQL;
     if ( isset($this->principal->username)) $this->username = $this->principal->username;
     if ( isset($this->principal->by_email)) $this->by_email = true;
 
+    if ( $this->collection_type == 'principal' || $this->collection_type == 'email' ) {
+      $this->collection = $this->principal->AsCollection();
+    }
+    elseif( $this->collection_type == 'root' ) {
+      $this->collection = (object) array(
+                            'collection_id' => 0,
+                            'dav_name' => '/',
+                            'dav_etag' => md5($c->system_name),
+                            'is_calendar' => 'f',
+                            'is_principal' => 'f',
+                            'user_no' => 0,
+                            'dav_displayname' => $c->system_name,
+                            'created' => date('Ymd\THis')
+                          );
+    }
+    elseif( $this->collection_type == 'proxy' ) {
+      $this->collection = (object) array(
+          'dav_name' => $this->collection_path,
+          'is_calendar' => 'f',
+          'is_principal' => 't',
+          'is_proxy' => 't',
+          'proxy_type' => $this->proxy_type,
+          'dav_displayname' => sprintf('Proxy %s for %s', $this->proxy_type, $this->principal->username),
+          'collection_id' => 0,
+          'user_no' => $this->principal->user_no,
+          'username' => $this->principal->username,
+          'email' => $this->principal->email,
+          'created' => $this->principal->created,
+          'dav_etag' => $this->principal->created
+      );
+    }
+
     /**
     * Evaluate our permissions for accessing the target
     */
@@ -358,7 +420,9 @@ EOSQL;
       return;
     }
 
-    $permissions = array();
+    $this->permissions = array();
+
+    if ( $this->IsPublic() ) $this->permissions['read'] = 'read';
 
     /**
     * In other cases we need to query the database for permissions
@@ -366,7 +430,6 @@ EOSQL;
     $qry = new PgQuery( "SELECT get_permissions( ?, ? ) AS perm;", $session->user_no, $this->user_no);
     if ( $qry->Exec("caldav") && $permission_result = $qry->Fetch() ) {
       $permission_result = "!".$permission_result->perm; // We prepend something to ensure we get a non-zero position.
-      $this->permissions = array();
       if ( strpos($permission_result,"A") )
         $this->permissions['all'] = 'all';
       else {
@@ -391,8 +454,6 @@ EOSQL;
   * @todo This logic does not catch all locking scenarios.  For example an infinite
   * depth request should check the permissions for all collections and resources within
   * that.  At present we only maintain permissions on a per-collection basis though.
-  *
-  * @param string $dav_name The resource which we want to know the lock status for
   */
   function IsLocked() {
     if ( !isset($this->_locks_found) ) {
@@ -423,6 +484,17 @@ EOSQL;
   }
 
 
+  /**
+  * Checks whether the collection is public
+  */
+  function IsPublic() {
+    if ( isset($this->collection) && isset($this->collection->publicly_readable) && $this->collection->publicly_readable == 't' ) {
+      return true;
+    }
+    return false;
+  }
+
+
   /**
   * Returns the name for this depth: 0, 1, infinity
   */
@@ -473,7 +545,10 @@ EOSQL;
   * @param string $lock_token The opaquelocktoken which we are looking for
   */
   function ValidateLockToken( $lock_token ) {
-    if ( isset($this->lock_token) && $this->lock_token == $lock_token ) return true;
+    if ( isset($this->lock_token) && $this->lock_token == $lock_token ) {
+      dbg_error_log( "caldav", "They supplied a valid lock token.  Great!" );
+      return true;
+    }
     if ( isset($this->if_clause) ) {
       dbg_error_log( "caldav", "Checking lock token '%s' against '%s'", $lock_token, $this->if_clause );
       $tokens = preg_split( '/[<>]/', $this->if_clause );
@@ -568,6 +643,17 @@ EOSQL;
   }
 
 
+  /**
+  * Returns true if the URL referenced by this request is within a proxy URL
+  */
+  function IsProxyRequest( ) {
+    if ( !isset($this->_is_proxy_request) ) {
+      $this->_is_proxy_request = preg_match( '#^/[^/]+/calendar-proxy-(read|write)/?[^/]*$#', $this->path );
+    }
+    return $this->_is_proxy_request;
+  }
+
+
   /**
   * Returns true if the request asked for infinite depth
   */
@@ -688,6 +774,21 @@ EOSQL;
     @header( sprintf("HTTP/1.1 %d %s", $status, getStatusMessage($status)) );
     @header( sprintf("X-DAViCal-Version: DAViCal/%d.%d.%d; DB/%d.%d.%d", $c->code_major, $c->code_minor, $c->code_patch, $c->schema_major, $c->schema_minor, $c->schema_patch) );
     @header( "Content-type: ".$content_type );
+
+    if ( (isset($c->dbg['ALL']) && $c->dbg['ALL']) || (isset($c->dbg['response']) && $c->dbg['response']) ) {
+      $lines = headers_list();
+      dbg_error_log( "LOG ", "***************** Response Header ****************" );
+      foreach( $lines AS $v ) {
+        dbg_error_log( "LOG headers", "-->%s", $v );
+      }
+      dbg_error_log( "LOG ", "******************** Response ********************" );
+      // Log the request in all it's gory detail.
+      $lines = preg_split( '#[\r\n]+#', $message);
+      foreach( $lines AS $v ) {
+        dbg_error_log( "LOG response", "-->%s", $v );
+      }
+    }
+
     echo $message;
 
     if ( strlen($message) > 100 || strstr($message, "\n") ) {
