@@ -12,7 +12,7 @@ dbg_error_log("get", "GET method handler");
 
 require_once("iCalendar.php");
 
-if ( ! $request->AllowedTo('read') ) {
+if ( ! $request->AllowedTo('freebusy') ) {
   $request->DoResponse( 403, translate("You may not access that calendar") );
 }
 
@@ -20,6 +20,8 @@ if ( $request->IsCollection() ) {
   /**
   * The CalDAV specification does not define GET on a collection, but typically this is
   * used as a .ics download for the whole collection, which is what we do also.
+  *
+  * @todo Change this to reference the collection_id of the collection at this location.
   */
   $order_clause = ( isset($c->strict_result_ordering) && $c->strict_result_ordering ? " ORDER BY dav_id" : "");
   $qry = new PgQuery( "SELECT caldav_data, class, caldav_type, calendar_item.user_no, logged_user FROM caldav_data INNER JOIN calendar_item USING ( dav_id ) WHERE caldav_data.user_no = ? AND caldav_data.dav_name ~ ? $order_clause", $request->user_no, $request->path.'[^/]+$');
@@ -31,7 +33,7 @@ else {
 if ( !$qry->Exec("GET") ) {
   $request->DoResponse( 500, translate("Database Error") );
 }
-else if ( $qry->rows == 1 ) {
+else if ( $qry->rows == 1 && ! $request->IsCollection() ) {
   $event = $qry->Fetch();
 
   /** Default deny... */
@@ -45,31 +47,32 @@ else if ( $qry->rows == 1 ) {
     $allowed = true;
   }
   else if ( $event->class != 'PRIVATE' ) {
-    if ( $event->class == 'CONFIDENTIAL' && ! $request->AllowedTo('modify') ) {
-      // if the event is confidential we fake one that just says "Busy"
+    $allowed = true; // but we may well obfuscate it below
+    if ( ! $request->AllowedTo('read') || ( $event->class == 'CONFIDENTIAL' && ! $request->AllowedTo('modify') ) ) {
+      // The user is not admin / owner of this calendarlooking at his calendar and can not admin the other cal,
+      // or maybe they don't have *read* access but they got here, so they must at least have free/busy access
+      // so we will present an obfuscated version of the event that just says "Busy" (translated :-)
       $confidential = new iCalComponent();
-      $confidential->SetType($first->GetType());
+
+      $ical = new iCalComponent( $event->caldav_data );
+      $resources = $ical->GetComponents('VTIMEZONE',false);
+      $first = $resources[0];
+
+      $confidential->SetType($event->caldav_type);
       $confidential->AddProperty( 'SUMMARY', translate('Busy') );
       $confidential->AddProperty( 'CLASS', 'CONFIDENTIAL' );
       $confidential->SetProperties( $first->GetProperties('DTSTART'), 'DTSTART' );
       $confidential->SetProperties( $first->GetProperties('RRULE'), 'RRULE' );
       $confidential->SetProperties( $first->GetProperties('DURATION'), 'DURATION' );
       $confidential->SetProperties( $first->GetProperties('DTEND'), 'DTEND' );
+      $confidential->SetProperties( $first->GetProperties('UID'), 'UID' );
+      $confidential->SetProperties( $first->GetProperties('CREATED'), 'CREATED' );
 
-      $vcal = new iCalComponent();
-      $vcal->VCalendar();
-      $vcal->AddComponent($confidential);
-      $event->caldav_data = $vcal->Render();
-      $allowed = true;
-    }
-    else {
-      /**
-      * We don't do the hide_alarm nonsense here.  It only really ever applied to Mozilla,
-      * it's fixed in 0.8+ anyway and Mozilla doesn't do GET requests... :-)
-      */
-      $allowed = true;
+      $ical->SetComponents( array($confidential), $confidential->GetType() );
+      $event->caldav_data = $ical->Render();
     }
   }
+  // else $event->class == 'PRIVATE' and this person may not see it.
 
   if ( ! $allowed ) {
     $request->DoResponse( 403, translate("Forbidden") );
@@ -88,7 +91,9 @@ else {
   * the timezones that are referred to by the events we have selected.
   */
   $vcal = new iCalComponent();
-  $vcal->VCalendar( array("X-WR-CALNAME" => $request->collection->dav_displayname ) );
+  if ( isset($request->collection->dav_displayname) ) {
+    $vcal->VCalendar( array("X-WR-CALNAME" => $request->collection->dav_displayname) );
+  }
 
   $need_zones = array();
   $timezones = array();
@@ -124,17 +129,21 @@ else {
       /** No visibility even of the existence of these events if they aren't admin/owner/attendee */
       if ( $event->class == 'PRIVATE' ) continue;
 
-      // the user is not admin / owner of this calendarlooking at his calendar and can not admin the other cal
-      if ( $event->class == 'CONFIDENTIAL' ) {
-        // if the event is confidential we fake one that just says "Busy"
+      if ( ! $request->AllowedTo('read') || $event->class == 'CONFIDENTIAL' ) {
+        // The user is not admin / owner of this calendar looking at his calendar and can not admin the other cal,
+        // or maybe they don't have *read* access but they got here, so they must at least have free/busy access
+        // so we will present an obfuscated version of the event that just says "Busy" (translated :-)
         $confidential = new iCalComponent();
-        $confidential->SetType($first->GetType());
+        $confidential->SetType($resource->GetType());
         $confidential->AddProperty( 'SUMMARY', translate('Busy') );
         $confidential->AddProperty( 'CLASS', 'CONFIDENTIAL' );
-        $confidential->SetProperties( $first->GetProperties('DTSTART'), 'DTSTART' );
-        $confidential->SetProperties( $first->GetProperties('RRULE'), 'RRULE' );
-        $confidential->SetProperties( $first->GetProperties('DURATION'), 'DURATION' );
-        $confidential->SetProperties( $first->GetProperties('DTEND'), 'DTEND' );
+        $confidential->SetProperties( $resource->GetProperties('DTSTART'), 'DTSTART' );
+        $confidential->SetProperties( $resource->GetProperties('RRULE'), 'RRULE' );
+        $confidential->SetProperties( $resource->GetProperties('DURATION'), 'DURATION' );
+        $confidential->SetProperties( $resource->GetProperties('DTEND'), 'DTEND' );
+        $confidential->SetProperties( $resource->GetProperties('UID'), 'UID' );
+        $confidential->SetProperties( $resource->GetProperties('CREATED'), 'CREATED' );
+
 
         $vcal->AddComponent($confidential);
       }
