@@ -77,12 +77,8 @@ function handle_freebusy_request( $ic ) {
 
     // If we make it here, then it seems we are allowed to see their data...
     $where = " WHERE usr.email = ? AND collection.is_calendar ";
-    if ( isset( $fbq_start ) ) {
-      $where .= "AND (dtend >= ".qpg($fbq_start)."::timestamp with time zone ";
-      $where .= "OR calculate_later_timestamp(".qpg($fbq_start)."::timestamp with time zone,dtend,rrule) >= ".qpg($fbq_start)."::timestamp with time zone) ";
-    }
-    if ( isset( $fbq_end ) ) {
-      $where .= "AND dtstart <= ".qpg($fbq_end)."::timestamp with time zone ";
+    if ( isset( $fbq_start ) || isset( $fbq_end ) ) {
+      $where .= "AND rrule_event_overlaps( dtstart, dtend, rrule, ".qpg($fbq_start).", ".qpg($fbq_end)." ) ";
     }
     $where .= "AND caldav_data.caldav_type IN ( 'VEVENT', 'VFREEBUSY' ) ";
     $where .= "AND (calendar_item.transp != 'TRANSPARENT' OR calendar_item.transp IS NULL) ";
@@ -96,9 +92,10 @@ function handle_freebusy_request( $ic ) {
 
     $busy = array();
     $busy_tentative = array();
+    /** @TODO prove this is correct */
     $sql = "SELECT caldav_data.caldav_data, calendar_item.rrule, calendar_item.transp, calendar_item.status, ";
-    $sql .= "to_char(calendar_item.dtstart at time zone 'GMT',".iCalendar::SqlDateFormat().") AS start, ";
-    $sql .= "to_char(calendar_item.dtend at time zone 'GMT',".iCalendar::SqlDateFormat().") AS finish ";
+    $sql .= "to_char(calendar_item.dtstart at time zone 'GMT',".iCalendar::SqlUTCFormat().") AS start, ";
+    $sql .= "to_char(calendar_item.dtend at time zone 'GMT',".iCalendar::SqlUTCFormat().") AS finish ";
     $sql .= "FROM usr INNER JOIN collection USING (user_no) INNER JOIN caldav_data USING (collection_id) INNER JOIN calendar_item USING(dav_id)".$where;
     if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= " ORDER BY dav_id";
     $qry = new PgQuery( $sql, $attendee_email );
@@ -128,7 +125,7 @@ function handle_freebusy_request( $ic ) {
     $i = 0;
 
     $fb = new iCalComponent();
-    $fb->AddProperty( 'DTSTAMP',   date('Ymd\THis\Z') );
+    $fb->AddProperty( 'DTSTAMP',   gmdate('Ymd\THis\Z') );
     $fb->AddProperty( 'DTSTART',   $fbq_start );
     $fb->AddProperty( 'DTEND',     $fbq_end );
     $fb->AddProperty( 'UID',       $ic->GetPValue('UID') );
@@ -147,11 +144,12 @@ function handle_freebusy_request( $ic ) {
           if ( $date->GreaterThan($fbq_end) ) break;
           $todate = clone($date);
           $todate->AddDuration($duration);
-          $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $date->Render('Ymd\THis'), $todate->Render('Ymd\THis') ), $fbparams);
+          $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $date->RenderGMT(), $todate->RenderGMT() ), $fbparams);
         }
       }
       else {
-        $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $start->Render('Ymd\THis'), $v->finish ), $fbparams );
+        $finish = new iCalDate($v->finish);
+        $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $start->RenderGMT(), $finish->RenderGMT() ), $fbparams );
       }
     }
 
@@ -166,11 +164,12 @@ function handle_freebusy_request( $ic ) {
           if ( $date->GreaterThan($fbq_end) ) break;
           $todate = clone($date);
           $todate->AddDuration($duration);
-          $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $date->Render('Ymd\THis'), $todate->Render('Ymd\THis') ), $fbparams );
+          $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $date->RenderGMT(), $todate->RenderGMT() ), $fbparams );
         }
       }
       else {
-        $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $start->Render('Ymd\THis'), $v->finish ), $fbparams );
+        $finish = new iCalDate($v->finish);
+        $fb->AddProperty("FREEBUSY", sprintf("%s/%s", $start->RenderGMT(), $finish->RenderGMT() ), $fbparams );
       }
     }
 
@@ -185,11 +184,22 @@ function handle_freebusy_request( $ic ) {
     $responses[] = $response;
   }
 
-  $response = $reply->NewXMLElement( "schedule-response", $responses, $reply->GetXmlNsArray() );
+  $response = $reply->NewXMLElement( "schedule-response", $responses, $reply->GetXmlNsArray(), 'urn:ietf:params:xml:ns:caldav' );
   $request->XMLResponse( 200, $response );
 }
 
 
+function handle_cancel_request( $ic ) {
+  global $c, $session, $request;
+
+  $reply = new XMLDocument( array("DAV:" => "", "urn:ietf:params:xml:ns:caldav" => "C" ) );
+
+  $responses[] = $reply->NewXMLElement( "response", false, false, 'urn:ietf:params:xml:ns:caldav' );
+  $reply->CalDAVElement($response, "request-status", "2.0;Success" );  // Cargo-cult setting
+  $response = $reply->NewXMLElement( "schedule-response", $responses, $reply->GetXmlNsArray() );
+  $request->XMLResponse( 200, $response );
+}
+
 $ical = new iCalComponent( $request->raw_post );
 $method =  $ical->GetPValue('METHOD');
 
@@ -201,6 +211,11 @@ switch ( $method ) {
     handle_freebusy_request( $first );
     break;
 
+  case 'CANCEL':
+    dbg_error_log("POST", "Handling iTIP 'CANCEL'  method.", $method );
+    handle_cancel_request( $first );
+    break;
+
   default:
     dbg_error_log("POST", "Unhandled '%s' method in request.", $method );
 }
