@@ -59,6 +59,8 @@ foreach( $request->xml_tags AS $k => $v ) {
     case 'DAV::getcontentlanguage':             /** should return the user's chosen locale, or default locale */
     case 'DAV::current-user-privilege-set':     /** only vaguely supported */
     case 'DAV::allprop':                        /** limited support, needs to be checked for correctness at some point */
+    case 'DAV::group-member-set':               /** limited support, at the moment used for caldav proxy */
+    case 'DAV::group-membership':				/** limited support, at the moment used for caldav proxy */
 
     /**
     * Handled CalDAV properties
@@ -68,11 +70,14 @@ foreach( $request->xml_tags AS $k => $v ) {
     case 'urn:ietf:params:xml:ns:caldav:schedule-inbox-URL':               /** Support in development */
     case 'urn:ietf:params:xml:ns:caldav:schedule-outbox-URL':              /** Support in development */
     case 'urn:ietf:params:xml:ns:caldav:calendar-free-busy-set':           /** Deprecated, but should work fine */
+    case 'urn:ietf:params:xml:ns:caldav:supported-calendar-component-set': /** Should work fine */
 
     /**
     * Handled calendarserver properties
     */
     case 'http://calendarserver.org/ns/:getctag':                        /** Calendar Server extension like etag - should work fine (we just return etag) */
+    case 'http://calendarserver.org/ns/:calendar-proxy-read-for':	       /** Calendar Server Delegation readonly */
+    case 'http://calendarserver.org/ns/:calendar-proxy-write-for':       /** Calendar Server Delegation read-write */
 
       $prop_list[$ns_tag] = $ns_tag;
       dbg_error_log( "PROPFIND", "Adding attribute '%s'", $ns_tag );
@@ -80,12 +85,8 @@ foreach( $request->xml_tags AS $k => $v ) {
 
     /** fixed server definitions - should work fine */
     case 'DAV::supported-collation-set':
-    case 'DAV::supported-calendar-component-set':
     case 'DAV::principal-collection-set':
     case 'DAV::supported-privilege-set':
-
-//    case 'dropbox-home-URL':   // HTTP://CALENDARSERVER.ORG/NS/
-//    case 'notifications-URL':  // HTTP://CALENDARSERVER.ORG/NS/
       if ( $_SERVER['PATH_INFO'] == '/' || $_SERVER['PATH_INFO'] == '' ) {
         $arbitrary[$ns_tag] = $ns_tag;
         dbg_error_log( "PROPFIND", "Adding arbitrary DAV property '%s'", $ns_tag );
@@ -123,6 +124,8 @@ foreach( $request->xml_tags AS $k => $v ) {
     /**
     * Arbitrary DAV properties may also be reported
     */
+    case 'http://calendarserver.org/ns/:dropbox-home-URL':    // Not yet supported (what is it supposed to do?)
+    case 'http://calendarserver.org/ns/:notifications-URL':   // Not yet supported (what is it supposed to do?)
     case 'urn:ietf:params:xml:ns:caldav:calendar-description':        // Supported purely as an arbitrary property
     default:
       $arbitrary[$ns_tag] = $ns_tag;
@@ -131,6 +134,17 @@ foreach( $request->xml_tags AS $k => $v ) {
   }
 }
 
+ /**
+  * Returns a set of hrefs from a set of urls
+  */
+function href_set_from_paths( $path_set ) {
+  global $reply;
+  $href_set = array();
+  foreach ($path_set AS $k => $v) {
+     $href_set[] = $reply->href( $v );
+  }
+  return $href_set;
+}
 
 /**
 * Returns the array of privilege names converted into XMLElements
@@ -174,6 +188,8 @@ function add_arbitrary_properties(&$prop, $record) {
 function add_principal_properties( &$prop, &$denied ) {
   global $prop_list, $session, $c, $request, $reply;
 
+  dbg_error_log("PROPFIND", "Adding principal properties");
+
   $allprop = isset($prop_list['DAV::allprop']);
 
   if ( isset($prop_list['DAV::principal-URL'] ) ) {
@@ -184,12 +200,7 @@ function add_principal_properties( &$prop, &$denied ) {
   }
 
   if ( isset($prop_list['urn:ietf:params:xml:ns:caldav:calendar-home-set'] ) ) {
-    $home_set = array();
-    $chs = $request->principal->calendar_home_set;
-    foreach( $chs AS $k => $url ) {
-      $home_set[] = $reply->href( $url );
-    }
-    $reply->CalDAVElement( $prop, "calendar-home-set", $home_set );
+    $reply->CalDAVElement( $prop, "calendar-home-set", href_set_from_paths( $request->principal->calendar_home_set ) );
   }
   if ( isset($prop_list['urn:ietf:params:xml:ns:caldav:schedule-inbox-URL'] ) ) {
     $reply->CalDAVElement( $prop, "schedule-inbox-URL", $reply->href( $request->principal->schedule_inbox_url) );
@@ -197,21 +208,16 @@ function add_principal_properties( &$prop, &$denied ) {
   if ( isset($prop_list['urn:ietf:params:xml:ns:caldav:schedule-outbox-URL'] ) ) {
     $reply->CalDAVElement( $prop, "schedule-outbox-URL", $reply->href( $request->principal->schedule_outbox_url) );
   }
-
+/*  Not supported - don't pretend we do!
   if ( isset($prop_list['http://calendarserver.org/ns/:dropbox-home-URL'] ) ) {
     $reply->CalendarserverElement($prop, "dropbox-home-URL", $reply->href( $request->principal->dropbox_url) );
   }
   if ( isset($prop_list['http://calendarserver.org/ns/:notifications-URL'] ) ) {
     $reply->CalendarserverElement($prop, "notifications-URL", $reply->href( $request->principal->notifications_url) );
   }
-
+*/
   if ( isset($prop_list['urn:ietf:params:xml:ns:caldav:calendar-user-address-set'] ) ) {
-    $addr_set = array();
-    $uas = $request->principal->user_address_set;
-    foreach( $uas AS $k => $v ) {
-      $addr_set[] = $reply->href( $v );
-    }
-    $reply->CalDAVElement( $prop, "calendar-user-address-set", $addr_set );
+    $reply->CalDAVElement( $prop, "calendar-user-address-set", href_set_from_paths( $request->principal->user_address_set ) );
   }
 }
 
@@ -245,6 +251,17 @@ function add_general_properties( &$prop, &$denied, $record ) {
     $reply->DAVElement( $prop, "current-user-principal", $request->current_user_principal_xml);
   }
 
+ // caldav proxy
+  // as per 5.1 paragraph 5
+  // TODO: this duplicates code below. if possible, do said code only once.
+  if ( preg_match('#/[^/]+/calendar-proxy-(read|write)/?#', $record->dav_displayname, $matches) && isset($prop_list['DAV::group-member-set']) ) {
+  	if ($matches[1] == "read") {
+        $reply->DAVElement($prop, "group-member-set", href_set_from_paths( $request->principal->read_proxy_group ) );
+  	} else /* if ($matches[1] == "write") */ {
+  		$reply->DAVElement($prop, "group-member-set", href_set_from_paths( $request->principal->write_proxy_group ) );
+  	}
+  }
+
   if ( isset($prop_list['DAV::acl']) ) {
     /**
     * @todo This information is semantically valid but presents an incorrect picture.
@@ -329,6 +346,40 @@ function build_propstat_response( $prop, $denied, $url ) {
   return $response;
 }
 
+/**
+ * Add the calendar-proxy-read/write pseudocollections
+ * @param responses array of responses to which to add the collections
+ */
+function add_proxy_response( &$responses, $which, $parent_path ) {
+	global $request, $c, $session;
+
+  $collection = (object) '';
+  if ( $which == "read" ) {
+    $proxy_group = $request->principal->read_proxy_group;
+  } else if ( $which == "write" ) {
+    $proxy_group = $request->principal->write_proxy_group;
+  }
+  if ($parent_path != '/'.$request->principal->username.'/') {
+    return; // Nothing to proxy for
+  }
+
+  $collection->dav_name = $parent_path."calendar-proxy-".$which."/";
+  $collection->is_calendar  = 'f';
+  $collection->is_principal = 't';
+  $collection->is_proxy     = 't';
+  $collection->proxy_type   = $which;
+  $collection->dav_displayname = $collection->dav_name;
+  $collection->collection_id = 0;
+  $collection->user_no = $session->user_no;
+  $collection->username = $session->username;
+  $collection->email = $session->email;
+  $collection->created = date('Ymd\THis');
+  $collection->dav_etag = md5($c->system_name . $collection->dav_name . implode($proxy_group) );
+  $collection->proxy_for = $proxy_group;
+
+  $responses[] = collection_to_xml( $collection );
+}
+
 
 /**
 * Returns an XML sub-tree for a single collection record from the DB
@@ -336,7 +387,7 @@ function build_propstat_response( $prop, $denied, $url ) {
 function collection_to_xml( $collection ) {
   global $arbitrary, $prop_list, $session, $c, $request, $reply;
 
-  dbg_error_log("PROPFIND","Building XML Response for collection '%s'", $collection->dav_name );
+  dbg_error_log("PROPFIND","Building XML Response for collection '%s' (%d)", $collection->dav_name, $collection->collection_id );
 
   $allprop = isset($prop_list['DAV::allprop']);
 
@@ -360,9 +411,12 @@ function collection_to_xml( $collection ) {
     $prop->NewElement($reply->Caldav("supported-collation-set"), $collations );
   }
   if ( isset($prop_list['urn:ietf:params:xml:ns:caldav:supported-calendar-component-set']) ) {
+    // Note that this won't appear on a PROPFIND against a Principal URL, since this routine is only called for a collection
     $components = array();
-    $components[] = $reply->NewXMLElement( "comp", '', array("name" => "VEVENT"), false, false,'urn:ietf:params:xml:ns:caldav');
-    $components[] = $reply->NewXMLElement( "comp", '', array("name" => "VTODO"), false, false,'urn:ietf:params:xml:ns:caldav');
+    $set_of_components = array( "VEVENT", "VTODO", "VJOURNAL", "VTIMEZONE", "VFREEBUSY" );
+    foreach( $set_of_components AS $v ) {
+      $components[] = $reply->NewXMLElement( "comp", '', array("name" => $v), 'urn:ietf:params:xml:ns:caldav');
+    }
     $reply->CalDAVElement($prop, "supported-calendar-component-set", $components );
   }
   if ( $allprop || isset($prop_list['DAV::getcontenttype']) ) {
@@ -379,7 +433,7 @@ function collection_to_xml( $collection ) {
     if ( $collection->is_calendar == 't' ) {
       $resourcetypes[] = $reply->NewXMLElement( "calendar", false, false,'urn:ietf:params:xml:ns:caldav');
       $resourcetypes[] = $reply->NewXMLElement( "schedule-calendar", false, false, 'urn:ietf:params:xml:ns:caldav' );
-      $lqry = new PgQuery("SELECT sum(length(caldav_data)) FROM caldav_data WHERE user_no = ? AND dav_name ~ ?;", $collection->user_no, $collection->dav_name.'[^/]+$' );
+      $lqry = new PgQuery("SELECT sum(length(caldav_data)) FROM caldav_data WHERE user_no = ? AND dav_name ~ ?", $collection->user_no, $collection->dav_name.'[^/]+$' );
       if ( $lqry->Exec("PROPFIND",__LINE__,__FILE__) && $row = $lqry->Fetch() ) {
         $contentlength = $row->sum;
       }
@@ -393,6 +447,12 @@ function collection_to_xml( $collection ) {
     if ( $collection->is_principal == 't' ) {
       $resourcetypes[] = $reply->NewXMLElement( "principal", false, false, 'DAV:');
     }
+
+    if ( isset($collection->is_proxy) && $collection->is_proxy == 't' ) {
+      // As per Caldav Proxy 5.1 par. 3
+      $resourcetypes[] = $reply->NewXMLElement('calendar-proxy-'.$collection->proxy_type, false, false, 'http://calendarserver.org/ns/');
+    }
+
     if ( $allprop || isset($prop_list['DAV::getcontentlength']) ) {
       $reply->DAVElement( $prop, "getcontentlength", $contentlength );  // Not strictly correct as a GET on this URL would be longer
     }
@@ -401,6 +461,33 @@ function collection_to_xml( $collection ) {
     }
   }
 
+  if ( isset($collection->is_proxy) && $collection->is_proxy == 't' ) {
+    // Caldav proxy (not described in rfc, but CalendarServer has it)
+    if ( isset($prop_list['http://calendarserver.org/ns/:calendar-proxy-'.$collection->proxy_type.'-for'] ) ) {
+      if ( $collection->proxy_type == "read" ) {
+        $proxy_group = $request->principal->read_proxy_for;
+      } else if ( $collection->proxy_type == "write" ) {
+        $proxy_group = $request->principal->write_proxy_for;
+      }
+      $reply->CalendarserverElement($prop, 'calendar-proxy-'.$collection->proxy_type.'-for', $reply->href( $proxy_group ) );
+    }
+
+    if ( isset($prop_list['DAV::group-member-set']) ) {
+      if ( $collection->proxy_type == "read" ) {
+        $proxy_group = $request->principal->read_proxy_group;
+      } else if ( $collection->proxy_type == "write" ) {
+        $proxy_group = $request->principal->write_proxy_group;
+      }
+      $reply->DAVElement($prop, "group-member-set", $reply->href( $proxy_group ) );
+    }
+
+    if (isset($prop_list['DAV::group-membership'])) {
+      $reply->DAVElement($prop, "group-membership", $reply->href( $request->principal->group_membership ));
+    }
+
+  }
+
+
   if ( $allprop || isset($prop_list['DAV::displayname']) ) {
     $displayname = ( $collection->dav_displayname == "" ? ucfirst(trim(str_replace("/"," ", $collection->dav_name))) : $collection->dav_displayname );
     $reply->DAVElement( $prop, "displayname", $displayname );
@@ -411,16 +498,16 @@ function collection_to_xml( $collection ) {
   }
 
   if ( isset($prop_list['urn:ietf:params:xml:ns:caldav:calendar-free-busy-set'] ) ) {
-    if ( $collection->type == 'in' && $session->user_no == $collection->user_no ) {
+    if ( $session->user_no != $collection->user_no ) {
+      $reply->CalDAVElement( $denied, "calendar-free-busy-set");
+    }
+    else if ( $collection->type == 'in' ) {
       $fb_set = array();
       foreach( $request->principal->calendar_free_busy_set AS $k => $v ) {
         $fb_set[] = $reply->href( $v, false, "DAV:" );
       }
       $reply->CalDAVElement( $prop, "calendar-free-busy-set", $fb_set );
     }
-    else if ( $session->user_no != $collection->user_no ) {
-      $reply->CalDAVElement( $denied, "calendar-free-busy-set");
-    }
   }
 
   /**
@@ -492,13 +579,14 @@ function item_to_xml( $item ) {
   return build_propstat_response( $prop, $denied, $url );
 }
 
+
 /**
 * Get XML response for items in the collection
 * If '/' is requested, a list of visible users is given, otherwise
 * a list of calendars for the user which are parented by this path.
 */
 function get_collection_contents( $depth, $user_no, $collection ) {
-  global $session, $request, $reply, $prop_list;
+  global $session, $request, $reply, $prop_list, $arbitrary;
 
   dbg_error_log("PROPFIND","Getting collection contents: Depth %d, User: %d, Path: %s", $depth, $user_no, $collection->dav_name );
 
@@ -508,18 +596,24 @@ function get_collection_contents( $depth, $user_no, $collection ) {
     * Calendar collections may not contain calendar collections.
     */
     if ( $collection->dav_name == '/' ) {
-      $sql = "SELECT usr.*, '/' || username || '/' AS dav_name, md5( '/' || username || '/') AS dav_etag, ";
-      $sql .= "to_char(updated at time zone 'GMT',?) AS created, ";
+      $sql = "SELECT usr.*, '/' || username || '/' AS dav_name, md5(username || updated::text) AS dav_etag, ";
+      $sql .= "to_char(joined at time zone 'GMT',?) AS created, ";
       $sql .= "to_char(updated at time zone 'GMT',?) AS modified, ";
-      $sql .= "fullname AS dav_displayname, FALSE AS is_calendar, TRUE AS is_principal FROM usr ";
-      $sql .= "WHERE get_permissions($session->user_no,user_no) ~ '[RAW]';";
+      $sql .= "fullname AS dav_displayname, FALSE AS is_calendar, TRUE AS is_principal, ";
+      $sql .= "0 AS collection_id ";
+      $sql .= "FROM usr ";
+      $sql .= "WHERE get_permissions($session->user_no,user_no) ~ '[RAW]' ";
+      $sql .= "ORDER BY user_no";
     }
     else {
       $sql = "SELECT user_no, dav_name, dav_etag, ";
       $sql .= "to_char(created at time zone 'GMT',?) AS created, ";
       $sql .= "to_char(modified at time zone 'GMT',?) AS modified, ";
-      $sql .= "dav_displayname, is_calendar, FALSE AS is_principal FROM collection ";
+      $sql .= "dav_displayname, is_calendar, FALSE AS is_principal, ";
+      $sql .= "collection_id ";
+      $sql .= "FROM collection ";
       $sql .= "WHERE parent_container=".qpg($collection->dav_name);
+      $sql .= " ORDER BY collection_id";
     }
     $qry = new PgQuery($sql, PgQuery::Plain(iCalendar::HttpDateFormat()), PgQuery::Plain(iCalendar::HttpDateFormat()));
 
@@ -527,7 +621,7 @@ function get_collection_contents( $depth, $user_no, $collection ) {
       while( $subcollection = $qry->Fetch() ) {
         if ( $subcollection->is_principal == "t" ) {
           $principal = new CalDAVPrincipal($subcollection);
-          $responses[] = $principal->RenderAsXML($prop_list, &$reply);
+          $responses[] = $principal->RenderAsXML(array_merge($prop_list,$arbitrary), &$reply);
         }
         else {
           $responses[] = collection_to_xml( $subcollection );
@@ -537,6 +631,12 @@ function get_collection_contents( $depth, $user_no, $collection ) {
         }
       }
     }
+    if ( $collection->is_principal == "t" ) {
+      // Caldav Proxy: 5.1 par. 2: Add child resources calendar-proxy-(read|write)
+      dbg_error_log("PROPFIND","Adding calendar-proxy-read and write. Path: %s", $collection->dav_name);
+      add_proxy_response($responses, "read", $collection->dav_name);
+      add_proxy_response($responses, "write", $collection->dav_name);
+    }
   }
 
   /**
@@ -555,7 +655,7 @@ function get_collection_contents( $depth, $user_no, $collection ) {
     $sql .= "summary AS dav_displayname ";
     $sql .= "FROM caldav_data JOIN calendar_item USING( dav_id, user_no, dav_name) ";
     $sql .= "WHERE dav_name ~ ".qpg('^'.$collection->dav_name.'[^/]+$'). $privacy_clause;
-    if ( isset($c->strict_result_ordering) && $c->strict_result_ordering ) $sql .= " ORDER BY dav_id";
+    $sql .= "ORDER BY dav_name";
     $qry = new PgQuery($sql, PgQuery::Plain(iCalendar::HttpDateFormat()), PgQuery::Plain(iCalendar::HttpDateFormat()));
     if( $qry->Exec("PROPFIND",__LINE__,__FILE__) && $qry->rows > 0 ) {
       while( $item = $qry->Fetch() ) {
@@ -573,17 +673,24 @@ function get_collection_contents( $depth, $user_no, $collection ) {
 * subsidiary collections will also be got up to $depth
 */
 function get_collection( $depth, $user_no, $collection_path ) {
-  global $c, $request;
+  global $session, $c, $request, $prop_list, $arbitrary, $reply;
   $responses = array();
 
   dbg_error_log("PROPFIND","Getting collection: Depth %d, User: %d, Path: %s", $depth, $user_no, $collection_path );
 
+  if (preg_match('#/[^/]+/calendar-proxy-(read|write)/?#',$collection_path, $match) ) {
+  	// this should be a direct query to /<somewhere>/calendar-proxy-<something>
+  	dbg_error_log("PROPFIND","Simulating calendar-proxy-read or write. Path: %s", $collection_path);
+       add_proxy_response($responses, $match[1], $collection_path);
+  }
+
   if ( $collection_path == null || $collection_path == '/' || $collection_path == '' ) {
     $collection->dav_name = $collection_path;
     $collection->dav_etag = md5($c->system_name . $collection_path);
     $collection->is_calendar = 'f';
     $collection->is_principal = 'f';
     $collection->user_no = 0;
+    $collection->collection_id = 0;
     $collection->dav_displayname = $c->system_name;
     $collection->created = date('Ymd\THis');
     $responses[] = collection_to_xml( $collection );
@@ -591,22 +698,34 @@ function get_collection( $depth, $user_no, $collection_path ) {
   else {
     $user_no = intval($user_no);
     if ( preg_match( '#^/[^/]+/$#', $collection_path) ) {
-      $sql = "SELECT user_no, '/' || username || '/' AS dav_name, md5( '/' || username || '/') AS dav_etag, ";
-      $sql .= "to_char(updated at time zone 'GMT',?) AS created, ";
+      $sql = "SELECT usr.*, '/' || username || '/' AS dav_name, md5( username || updated::text ) AS dav_etag, ";
+      $sql .= "to_char(joined at time zone 'GMT',?) AS created, ";
       $sql .= "to_char(updated at time zone 'GMT',?) AS modified, ";
-      $sql .= "fullname AS dav_displayname, FALSE AS is_calendar, TRUE AS is_principal FROM usr WHERE user_no = $user_no ; ";
+      $sql .= "fullname AS dav_displayname, FALSE AS is_calendar, TRUE AS is_principal, 0 AS collection_id ";
+      $sql .= "FROM usr WHERE user_no = $user_no ";
+      $sql .= "AND get_permissions($session->user_no,user_no) ~ '[RAW]' ";
+      $sql .= "ORDER BY user_no";
     }
     else {
       $sql = "SELECT user_no, dav_name, dav_etag, ";
       $sql .= "to_char(created at time zone 'GMT',?) AS created, ";
       $sql .= "to_char(modified at time zone 'GMT',?) AS modified, ";
-      $sql .= "dav_displayname, is_calendar, FALSE AS is_principal FROM collection WHERE user_no = $user_no AND dav_name = ".qpg($collection_path);
+      $sql .= "dav_displayname, is_calendar, FALSE AS is_principal, collection_id ";
+      $sql .= "FROM collection WHERE dav_name = ".qpg($collection_path);
+      $sql .= " ORDER BY collection_id";
     }
     $qry = new PgQuery($sql, PgQuery::Plain(iCalendar::HttpDateFormat()), PgQuery::Plain(iCalendar::HttpDateFormat()) );
     if( $qry->Exec("PROPFIND",__LINE__,__FILE__) && $qry->rows > 0 && $collection = $qry->Fetch() ) {
-      $responses[] = collection_to_xml( $collection );
+      if ( $collection->is_principal == "t" ) {
+        $principal = new CalDAVPrincipal($collection);
+        $responses[] = $principal->RenderAsXML(array_merge($prop_list,$arbitrary), &$reply);
+      }
+      else {
+        $responses[] = collection_to_xml( $collection );
+      }
+
     }
-    elseif ( $c->collections_always_exist ) {
+    elseif ( $c->collections_always_exist && preg_match( "#^/$session->username/#", $collection_path) ) {
       dbg_error_log("PROPFIND","Using $c->collections_always_exist setting is deprecated" );
       $collection->dav_name = $collection_path;
       $collection->dav_etag = md5($collection_path);
@@ -661,7 +780,19 @@ $request->UnsupportedRequest($unsupported); // Won't return if there was unsuppo
 */
 $url = ConstructURL( $request->path );
 $url = preg_replace( '#/$#', '', $url);
-if ( $request->IsCollection() ) {
+$responses = array();
+if ( $request->IsPrincipal() ) {
+  $responses[] = $request->principal->RenderAsXML(array_merge($prop_list,$arbitrary), &$reply);
+  if ( $request->depth > 0 ) {
+    $collection = (object) array( 'dav_name' => '/'.$request->username.'/', 'is_calendar' => 'f', 'is_principal' => 't' );
+    $responses = array_merge($responses, get_collection_contents( $request->depth - 1,  $request->user_no, $collection ) );
+  }
+}
+if ( $request->IsProxyRequest() ) {
+  add_proxy_response($responses, $request->proxy_type, '/' . $request->principal->username . '/' );
+  /** Nothing inside these, as yet. */
+}
+elseif ( $request->IsCollection() ) {
   $responses = get_collection( $request->depth, $request->user_no, $request->path );
 }
 elseif ( $request->AllowedTo('read') ) {
