@@ -20,7 +20,7 @@ include_once("iCalendar.php");
 /**
 * A regex which will match most reasonable timezones acceptable to PostgreSQL.
 */
-$tz_regex = ':^(Africa|America|Antarctica|Arctic|Asia|Atlantic|Australia|Brazil|Canada|Chile|Etc|Europe|Indian|Mexico|Mideast|Pacific|US)/[a-z]+$:i';
+$tz_regex = ':^(Africa|America|Antarctica|Arctic|Asia|Atlantic|Australia|Brazil|Canada|Chile|Etc|Europe|Indian|Mexico|Mideast|Pacific|US)/[a-z_]+$:i';
 
 /**
 * This function launches an error
@@ -127,10 +127,25 @@ function public_events_only( $user_no, $dav_name ) {
 
 
 /**
-* Create scheduling requests in the schedule inbox for the
-* @param iCalComponent $component The VEVENT/VTODO/... resource we are scheduling
+* Create a scheduling request in the schedule inbox for the
+* @param iCalComponent $resource The VEVENT/VTODO/... resource we are scheduling
+* @param iCalProp $attendee The attendee we are scheduling
+* @return float The result of the scheduling request, per caldav-sched #3.5.4
 */
-function create_scheduling_requests( $resource ) {
+function write_scheduling_request( &$resource, $attendee ) {
+  return '5.4';
+}
+
+/**
+* Create scheduling requests in the schedule inbox for the
+* @param iCalComponent $resource The VEVENT/VTODO/... resource we are scheduling
+*/
+function create_scheduling_requests( &$resource ) {
+  if ( ! is_object($resource) ) {
+    dbg_error_log( "PUT", "create_scheduling_requests called with non-object parameter (%s)", gettype($resource) );
+    return;
+  }
+
   $attendees = $resource->GetPropertiesByPath('/VCALENDAR/*/ATTENDEE');
   if ( preg_match( '# iCal/\d#', $_SERVER['HTTP_USER_AGENT']) ) {
     dbg_error_log( "POST", "Non-compliant iCal request.  Using X-WR-ATTENDEE property" );
@@ -139,12 +154,48 @@ function create_scheduling_requests( $resource ) {
       $attendees[] = $v;
     }
   }
+  if ( count($attendees) == 0 ) {
+    dbg_error_log( "PUT", "Event has no attendees - no scheduling required.", count($attendees) );
+    return;
+  }
+
   dbg_error_log( "PUT", "Adding to scheduling inbox %d attendees", count($attendees) );
   foreach( $attendees AS $attendee ) {
-    dbg_error_log( "PUT", "Not yet adding to schedule-inbox for %s", $attendee->Value() );
+    $attendee->SetParameterValue( 'SCHEDULE-STATUS', write_scheduling_request( $resource, $attendee->Value() ) );
   }
 }
 
+
+/**
+* Update scheduling requests in the schedule inbox for the
+* @param iCalComponent $resource The VEVENT/VTODO/... resource we are scheduling
+*/
+function update_scheduling_requests( &$resource ) {
+  if ( ! is_object($resource) ) {
+    dbg_error_log( "PUT", "update_scheduling_requests called with non-object parameter (%s)", gettype($resource) );
+    return;
+  }
+
+  $attendees = $resource->GetPropertiesByPath('/VCALENDAR/*/ATTENDEE');
+  if ( preg_match( '# iCal/\d#', $_SERVER['HTTP_USER_AGENT']) ) {
+    dbg_error_log( "POST", "Non-compliant iCal request.  Using X-WR-ATTENDEE property" );
+    $wr_attendees = $resource->GetPropertiesByPath('/VCALENDAR/*/X-WR-ATTENDEE');
+    foreach( $wr_attendees AS $k => $v ) {
+      $attendees[] = $v;
+    }
+  }
+  if ( count($attendees) == 0 ) {
+    dbg_error_log( "PUT", "Event has no attendees - no scheduling required.", count($attendees) );
+    return;
+  }
+
+  dbg_error_log( "PUT", "Adding to scheduling inbox %d attendees", count($attendees) );
+  foreach( $attendees AS $attendee ) {
+    $attendee->SetParameterValue( 'SCHEDULE-STATUS', write_scheduling_request( $resource, $attendee->Value() ) );
+  }
+}
+
+
 /**
 * This function will import a whole calendar
 * @param string $ics_content the ics file to import
@@ -208,6 +259,7 @@ function import_collection( $ics_content, $user_no, $path, $caldav_context ) {
     $vcal = new iCalComponent();
     $vcal->VCalendar();
     $vcal->SetComponents($resource);
+    create_scheduling_requests($vcal);
     $icalendar = $vcal->Render();
 
     /** As ever, we mostly deal with the first resource component */
@@ -284,12 +336,12 @@ function import_collection( $ics_content, $user_no, $path, $caldav_context ) {
         unset($tz);
         unset($tz_locn);
       }
-      dbg_error_log( "PUT", " Using TZID[%s] and location of [%s]", $tzid, (isset($tz_locn) ? $tz_locn : '') );
       if ( ! isset($tz_locn) || ! preg_match( $tz_regex, $tz_locn ) ) {
-        if ( preg_match( '#/([^/]+/[^/]+)$#', $tzid, $matches ) ) {
+        if ( preg_match( '#([^/]+/[^/]+)$#', $tzid, $matches ) ) {
           $tz_locn = $matches[1];
         }
       }
+      dbg_error_log( "PUT", " Using TZID[%s] and location of [%s]", $tzid, (isset($tz_locn) ? $tz_locn : '') );
       if ( isset($tz_locn) && ($tz_locn != $last_tz_locn) && preg_match( $tz_regex, $tz_locn ) ) {
         dbg_error_log( "PUT", " Setting timezone to %s", $tz_locn );
         $sql .= ( $tz_locn == '' ? '' : "SET TIMEZONE TO ".qpg($tz_locn).";" );
@@ -406,11 +458,13 @@ function putCalendarResource( &$request, $author, $caldav_context ) {
   $first = $resources[0];
 
   if ( $put_action_type == 'INSERT' ) {
+    create_scheduling_requests($vcal);
     $qry = new PgQuery( "BEGIN; INSERT INTO caldav_data ( user_no, dav_name, dav_etag, caldav_data, caldav_type, logged_user, created, modified, collection_id ) VALUES( ?, ?, ?, ?, ?, ?, current_timestamp, current_timestamp, ? )",
                            $request->user_no, $request->path, $etag, $request->raw_post, $first->GetType(), $author, $request->collection_id );
     if ( !$qry->Exec("PUT") ) rollback_on_error( $caldav_context, $request->user_no, $request->path);
   }
   else {
+    update_scheduling_requests($vcal);
     $qry = new PgQuery( "BEGIN;UPDATE caldav_data SET caldav_data=?, dav_etag=?, caldav_type=?, logged_user=?, modified=current_timestamp WHERE user_no=? AND dav_name=?",
                            $request->raw_post, $etag, $first->GetType(), $author, $request->user_no, $request->path );
     if ( !$qry->Exec("PUT") ) rollback_on_error( $caldav_context, $request->user_no, $request->path);
@@ -492,20 +546,33 @@ function putCalendarResource( &$request, $author, $caldav_context ) {
   if ( !isset($tzid) || $tzid == "" ) $tzid = $first->GetPParamValue('DUE','TZID');
   $timezones = $ic->GetComponents('VTIMEZONE');
   foreach( $timezones AS $k => $tz ) {
-    if ( $tz->GetPValue('TZID') == $tzid ) {
-      // This is the one
-      $tz_locn = $tz->GetPValue('X-LIC-LOCATION');
-      dbg_error_log( "PUT", " Using TZID[%s] and location of [%s]", $tzid, $tz_locn );
-      if ( ! isset($tz_locn) || ! preg_match( $tz_regex, $tz_locn ) ) {
-        if ( preg_match( '#/([^/]+/[^/]+)$#', $tzid, $matches ) ) {
-          $tz_locn = $matches[1];
-        }
+    if ( $tz->GetPValue('TZID') != $tzid ) {
+      /**
+      * We'll pretend they didn't forget to give us a TZID and that they
+      * really hope the server is running in the timezone they supplied... but be noisy about it.
+      */
+      dbg_error_log( "ERROR", " Event includes TZID[%s] but users TZID[%s]!", $tz->GetPValue('TZID'), $tzid );
+      $tzid = $tz->GetPValue('TZID');
+    }
+    // This is the one
+    $tz_locn = $tz->GetPValue('X-LIC-LOCATION');
+    if ( ! isset($tz_locn) ) {
+      if ( preg_match( '#([^/]+/[^/]+)$#', $tzid, $matches ) )
+        $tz_locn = $matches[1];
+      else {
+        dbg_error_log( "ERROR", " Couldn't guess Olsen TZ from TZID[%s].  This may end in tears...", $tzid );
       }
-      if ( isset($tz_locn) && preg_match( $tz_regex, $tz_locn ) ) {
-        dbg_error_log( "PUT", " Setting timezone to %s", $tz_locn );
-        $sql = ( $tz_locn == '' ? '' : "SET TIMEZONE TO ".qpg($tz_locn).";" );
+    }
+    else {
+      if ( ! preg_match( $tz_regex, $tz_locn ) ) {
+        if ( preg_match( '#([^/]+/[^/]+)$#', $tzid, $matches ) ) $tz_locn = $matches[1];
       }
     }
+    dbg_error_log( "PUT", " Using TZID[%s] and location of [%s]", $tzid, $tz_locn );
+    if ( isset($tz_locn) && preg_match( $tz_regex, $tz_locn ) ) {
+      dbg_error_log( "PUT", " Setting timezone to %s", $tz_locn );
+      $sql = ( $tz_locn == '' ? '' : "SET TIMEZONE TO ".qpg($tz_locn).";" );
+    }
     $qry = new PgQuery("SELECT tz_locn FROM time_zone WHERE tz_id = ?", $tzid );
     if ( $qry->Exec() && $qry->rows == 0 ) {
       $qry = new PgQuery("INSERT INTO time_zone (tz_id, tz_locn, tz_spec) VALUES(?,?,?)", $tzid, $tz_locn, $tz->Render() );
