@@ -80,7 +80,7 @@ function calendar_to_xml( $properties, $item ) {
   if ( isset($properties['calendar-data']) || isset($properties['displayname']) ) {
     if ( !$request->AllowedTo('all') && $session->user_no != $item->user_no ){
       // the user is not admin / owner of this calendarlooking at his calendar and can not admin the other cal
-      /** @todo We should examine the ORGANIZER and ATTENDEE fields in the event.  If this person is there then they should see this, and perhaps get alarms also */
+      /** @todo We should examine the ORGANIZER and ATTENDEE fields in the event.  If this person is there then they should see this */
       if ( $item->class == 'CONFIDENTIAL' ) {
         $ical = new iCalComponent( $caldav_data );
         $resources = $ical->GetComponents('VTIMEZONE',false);
@@ -95,21 +95,11 @@ function calendar_to_xml( $properties, $item ) {
         $confidential->SetProperties( $first->GetProperties('RRULE'), 'RRULE' );
         $confidential->SetProperties( $first->GetProperties('DURATION'), 'DURATION' );
         $confidential->SetProperties( $first->GetProperties('DTEND'), 'DTEND' );
+        $confidential->SetProperties( $first->GetProperties('UID'), 'UID' );
         $ical->SetComponents(array($confidential),$confidential->GetType());
 
         $caldav_data = $ical->Render();
       }
-      elseif ( isset($c->hide_alarm) && $c->hide_alarm ) {
-        // Otherwise we hide the alarms (if configured to)
-        $ical = new iCalComponent( $caldav_data );
-        $timezones = $ical->GetComponents('VTIMEZONE',true);
-        $resources = $ical->GetComponents('VTIMEZONE',false);
-        foreach( $resources AS $k => $v ) {
-          $v->ClearComponents('VALARM');
-        }
-        $ical->SetComponents($timezones + $resources);
-        $caldav_data = $ical->render(true, $caldav_type );
-      }
     }
   }
 
@@ -128,28 +118,15 @@ function calendar_to_xml( $properties, $item ) {
       case 'getcontenttype':
         $prop->NewElement($k, "text/calendar" );
         break;
-/*
-* I don't think this is correct.  We should only list these properties against
-* the (relevant) collection, not against it's contents.
-      case 'resourcetype':
-        $prop->NewElement($k, new XMLElement($reply->Caldav("calendar"), false) );
-        if ( $request->collection_type == 'in' ) {
-          $prop->NewElement($k, new XMLElement($reply->Caldav("schedule-inbox"), false) );
-        }
-        else if ( $request->collection_type == 'out' ) {
-          $prop->NewElement($k, new XMLElement($reply->Caldav("schedule-outbox"), false) );
-        }
-        else {
-          $prop->NewElement($k, new XMLElement($reply->Caldav("schedule-calendar"), false) );
-        }
-        break;
-*/
       case 'current-user-principal':
         $prop->NewElement("current-user-principal", $request->current_user_principal_xml);
         break;
       case 'displayname':
         $prop->NewElement($k, $displayname );
         break;
+      case 'resourcetype':
+        $prop->NewElement($k); // Just an empty resourcetype for a non-collection.
+        break;
       case 'getetag':
         $prop->NewElement($k, '"'.$item->dav_etag.'"' );
         break;
